"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.ProcessLauncher = void 0;

var _splice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/splice"));

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _some = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/some"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/keys"));

var _isArray = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/is-array"));

var _fsExtra = require("fs-extra");

var _embarkUtils = require("embark-utils");

const child_process = require('child_process');

const path = require('path');

const constants = (0, _fsExtra.readJsonSync)(path.join(__dirname, '../../constants.json'));
let processCount = 1;

class ProcessLauncher {
  /**
   * Constructor of ProcessLauncher. Forks the module and sets up the message handling
   * @param {Object}    options   Options tp start the process
   *        * modulePath      {String}    Absolute path to the module to fork
   *        * logger          {Object}    Logger
   *        * events          {Function}  Events Emitter instance
   * @return {ProcessLauncher}    The ProcessLauncher instance
   */
  constructor(options) {
    this.name = options.name || path.basename(options.modulePath);

    if ((0, _embarkUtils.isDebug)()) {
      const childOptions = {
        stdio: 'pipe',
        execArgv: ['--inspect-brk=' + (60000 + processCount)]
      };
      processCount++;
      this.process = child_process.fork(options.modulePath, [], childOptions);
    } else {
      this.process = child_process.fork(options.modulePath);
    }

    this.logger = options.logger;
    this.events = options.events;
    this.silent = options.silent;
    this.exitCallback = options.exitCallback;
    this.embark = options.embark;
    this.logs = [];
    this.events.request('process:logs:register', {
      processName: this.name,
      eventName: `process:${this.name}:log`,
      silent: this.silent
    });
    this.subscriptions = {};

    this._subscribeToMessages();
  }

  setSilent(value) {
    if (this.silent === value) {
      return;
    }

    this.silent = value;
    this.events.request('process:logs:register', {
      processName: this.name,
      eventName: `process:${this.name}:log`,
      silent: this.silent
    });
  } // Subscribes to messages from the child process and delegates to the right methods


  _subscribeToMessages() {
    const self = this;
    this.process.on('message', msg => {
      if (msg.error) {
        self.logger.error(msg.error);
      }

      if (msg.result === constants.process.log) {
        return this.events.emit(`process:${this.name}:log`, msg.type, msg.message);
      }

      if (msg.event) {
        return self._handleEvent(msg);
      }

      self._checkSubscriptions(msg);
    });
    this.process.on('exit', code => {
      if (self.exitCallback) {
        return self.exitCallback(code);
      }

      if (code) {
        this.logger.info(`Child Process ${this.name} exited with code ${code}`);
      }
    });
  } // Handle event calls from the child process


  _handleEvent(msg) {
    const self = this;

    if (!self.events[msg.event]) {
      self.logger.warn('Unknown event method called: ' + msg.event);
      return;
    }

    if (!msg.args || !(0, _isArray.default)(msg.args)) {
      msg.args = [];
    } // Add callback in the args


    msg.args.push(result => {
      self.process.send({
        event: constants.process.events.response,
        result,
        eventId: msg.eventId
      });
    });
    self.events[msg.event](msg.requestName, ...msg.args);
  } // Looks at the subscriptions to see if there is a callback to call


  _checkSubscriptions(msg) {
    const messageKeys = (0, _keys.default)(msg);
    const subscriptionsKeys = (0, _keys.default)(this.subscriptions);
    let subscriptionsForKey;
    let messageKey; // Find if the message contains a key that we are subscribed to

    (0, _some.default)(messageKeys).call(messageKeys, _messageKey => {
      return (0, _some.default)(subscriptionsKeys).call(subscriptionsKeys, subscriptionKey => {
        if (_messageKey === subscriptionKey) {
          subscriptionsForKey = this.subscriptions[subscriptionKey];
          messageKey = _messageKey;
          return true;
        }

        return false;
      });
    });

    if (subscriptionsForKey) {
      // Find if we are subscribed to one of the values
      let subsIndex = [];
      const subscriptionsForValue = (0, _filter.default)(subscriptionsForKey).call(subscriptionsForKey, (sub, index) => {
        if (msg[messageKey] === sub.value) {
          subsIndex.push(index);
          return true;
        }

        return false;
      });

      if (subscriptionsForValue.length) {
        // We are subscribed to that message, call the callback
        (0, _forEach.default)(subscriptionsForValue).call(subscriptionsForValue, (subscription, index) => {
          subscription.callback(msg);

          if (subscription.once) {
            var _context;

            // Called only once, we can remove it
            subscription = null;
            (0, _splice.default)(_context = this.subscriptions[messageKey]).call(_context, subsIndex[index], 1);
          }
        });
      }
    }
  }
  /**
   * Subscribe to a message using a key-value pair
   * @param {String}    key       Message key to subscribe to
   * @param {String}    value     Value that the above key must have for the callback to be called
   * @param {Function}  callback  callback(response)
   * @return {void}
   */


  on(key, value, callback) {
    if (this.subscriptions[key]) {
      this.subscriptions[key].push({
        value,
        callback
      });
      return;
    }

    this.subscriptions[key] = [{
      value,
      callback
    }];
  }
  /**
   * Same as .on, but only triggers once
   * @param {String}    key       Message key to subscribe to
   * @param {String}    value     Value that the above key must have for the callback to be called
   * @param {Function}  callback  callback(response)
   * @return {void}
   */


  once(key, value, callback) {
    const obj = {
      value,
      callback,
      once: true
    };

    if (this.subscriptions[key]) {
      this.subscriptions[key].push(obj);
      return;
    }

    this.subscriptions[key] = [obj];
  }
  /**
   * Unsubscribes from a previously subscribed key-value pair (or key if no value)
   * @param {String}  key     Message key to unsubscribe
   * @param {String}  value   [Optional] Value of the key to unsubscribe
   *                            If there is no value, unsubscribes from all the values of that key
   * @return {void}
   */


  unsubscribeTo(key, value) {
    if (!value) {
      this.subscriptions[key] = [];
    }

    if (this.subscriptions[key]) {
      var _context2;

      (0, _filter.default)(_context2 = this.subscriptions[key]).call(_context2, (val, index) => {
        if (val.value === value) {
          var _context3;

          (0, _splice.default)(_context3 = this.subscriptions[key]).call(_context3, index, 1);
        }
      });
    }
  }
  /**
   * Unsubscribes from all subscriptions
   * @return {void}
   */


  unsubscribeToAll() {
    this.subscriptions = {};
  }
  /**
   * Sends a message to the child process. Same as ChildProcess.send()
   * @params {Object}   message     Message to send
   * For other parameters, see:
   *  https://nodejs.org/api/child_process.html#child_process_subprocess_send_message_sendhandle_options_callback
   * @return {void}
   */


  send() {
    if (!this.process.connected) {
      return false;
    }

    return this.process.send(...arguments);
  }
  /**
   * Disconnects the child process. It will exit on its own
   * @return {void}
   */


  disconnect() {
    this.process.disconnect();
  }
  /**
   * Kills the child process
   *  https://nodejs.org/api/child_process.html#child_process_subprocess_kill_signal
   * @return {void}
   */


  kill() {
    this.process.kill(...arguments);
  }

}

exports.ProcessLauncher = ProcessLauncher;
//# sourceMappingURL=processLauncher.js.map